import { nip19, Event } from 'nostr-tools'

/**
 * Get the current time in seconds since the Unix epoch (January 1, 1970).
 *
 * This function retrieves the current time in milliseconds since the Unix epoch
 * using `Date.now()` and then converts it to seconds by dividing by 1000.
 * Finally, it rounds the result to the nearest whole number using `Math.round()`.
 *
 * @returns {number} The current time in seconds since the Unix epoch.
 */
export const now = (): number => Math.round(Date.now() / 1000)

/**
 * Converts a hexadecimal public key to an npub format.
 * If the input is already in npub format, it returns the input unchanged.
 *
 * @param {string} hexPubkey - The hexadecimal public key to be converted.
 * @returns {`npub1${string}`} - The public key in npub format.
 */
export const hexToNpub = (hexPubkey: string): `npub1${string}` => {
  // Check if the input is already in npub format and return it unchanged if true
  if (hexPubkey.startsWith('npub1')) return hexPubkey as `npub1${string}`

  // Convert the hexadecimal public key to npub format using the nip19 encoder
  return nip19.npubEncode(hexPubkey)
}

/**
 * Retrieves the value associated with a specific tag identifier from an event.
 *
 * This function searches the `tags` array of an event to find a tag that matches the given
 * `tagIdentifier`. If a matching tag is found, it returns the associated value(s).
 * If no matching tag is found, it returns `null`.
 *
 * @param event - The event object containing the tags.
 * @param tagIdentifier - The identifier of the tag to search for.
 * @returns {string | null} The value(s) associated with the specified tag identifier, or `null` if the tag is not found.
 */
export const getTagValue = (
  event: Event,
  tagIdentifier: string
): string[] | null => {
  // Find the tag in the event's tags array where the first element matches the tagIdentifier.
  const tag = event.tags.find((item) => item[0] === tagIdentifier)

  // If a matching tag is found, return the rest of the elements in the tag (i.e., the values).
  if (tag) {
    return tag.slice(1) // Slice to remove the identifier, returning only the values.
  }

  // Return null if no matching tag is found.
  return null
}

/**
 * @param hexKey hex private or public key
 * @returns whether or not is key valid
 */
const validateHex = (hexKey: string) => {
  return hexKey.match(/^[a-f0-9]{64}$/)
}

/**
 * NPUB provided - it will convert NPUB to HEX
 * HEX provided - it will return HEX
 *
 * @param pubKey in NPUB, HEX format
 * @returns HEX format
 */
export const npubToHex = (pubKey: string): string | null => {
  // If key is NPUB
  if (pubKey.startsWith('npub1')) {
    try {
      return nip19.decode(pubKey).data as string
    } catch (error) {
      return null
    }
  }

  // valid hex key
  if (validateHex(pubKey)) return pubKey

  // Not a valid hex key
  return null
}

/**
 * Extracts the zap amount from an event object.
 *
 * @param event - The event object from which the zap amount needs to be extracted.
 * @returns The zap amount in the form of a number, converted from the extracted data, or 0 if the amount cannot be determined.
 */
export const extractZapAmount = (event: Event): number => {
  // Find the 'description' tag within the event's tags
  const description = event.tags.find(
    (tag) => tag[0] === 'description' && typeof tag[1] === 'string'
  )

  // If the 'description' tag is found and it has a valid value
  if (description && description[1]) {
    try {
      // Parse the description as JSON to get additional details
      const parsedDescription: Event = JSON.parse(description[1])

      // Find the 'amount' tag within the parsed description's tags
      const amountTag = parsedDescription.tags.find(
        (tag) => tag[0] === 'amount' && typeof tag[1] === 'string'
      )

      // If the 'amount' tag is found and it has a valid value, convert it to an integer and return
      if (amountTag && amountTag[1]) return parseInt(amountTag[1]) / 1000
    } catch (error) {
      // Log an error message if JSON parsing fails
      console.log(
        `An error occurred while parsing description of zap event: ${error}`
      )
    }
  }

  // Return 0 if the zap amount cannot be determined
  return 0
}
